
# q-expansion of newform 4160.2.a.b, downloaded from the LMFDB on 16 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4160
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-2], [-1], [4], [-6], [-1], [-6], [2], [-6], [6], [-2], [-2], [-6], [2], [12], [-6], [6], [-2], [-4], [6], [-10], [4], [0], [-6], [2], [-6], [10], [18], [10], [-6], [-2], [-12], [-6], [-16], [-6], [10], [10], [20], [12], [18], [12], [-2], [0], [2], [-6], [16], [-4], [28], [-12], [-14], [18], [-30], [26], [24], [-6], [-18], [18], [-14], [10], [18], [2], [6], [8], [12], [26], [30], [14], [-22], [18], [-14], [6], [30], [10], [34], [2], [-12], [-6], [22], [18], [-22], [-24], [-38], [-18], [26], [16], [6], [-30], [26], [-6], [16], [-18], [6], [16], [-24], [-22], [6], [-30], [6], [-34], [-14], [-22], [6], [18], [6], [-40], [-34], [36], [-30], [12], [26], [-14], [10], [42], [14], [34], [-30], [32], [-18], [-6], [0], [10], [-22], [42], [12], [-10], [18], [-14], [0], [-2], [-14], [-34], [-36], [4], [-38], [-54], [-22], [-18], [8], [18], [30], [14], [18], [-14], [48], [46], [6], [-26], [18], [8], [-12], [-14], [54], [50], [42], [-10], [0], [-20], [18], [2], [-6], [-24], [-30], [-32], [-12], [30], [48], [16], [10], [26], [-6], [-30], [-14], [-60], [-34], [40], [42], [-22], [42], [34], [34], [28], [24], [34], [-6], [0], [-42], [-38], [8], [26], [-6], [26], [-24], [-28], [66], [6], [18], [-46], [34], [6], [6], [-30], [16], [10], [-58], [-42], [-30], [58], [-42], [-30], [-40], [-70], [-42], [46], [0], [66], [-46], [-38], [-30], [-24], [-30], [70], [-2], [66], [-20], [-42], [-26], [-6], [36], [-26], [18], [10], [56], [-50], [-6], [-34], [-24], [26], [54], [-60], [-24], [-24], [56], [-50], [-38], [-30], [-24], [-44], [42], [26], [66], [22], [18], [36], [26], [-78], [30], [-74], [-28], [-6], [38], [-62], [18], [34], [10], [-42], [-64], [18], [66], [-4], [18], [34], [-28], [26], [-32], [2], [-20], [12], [-62], [-22], [24], [54], [58], [66], [10], [32], [12], [74], [42], [22], [30], [-18], [18], [42], [18], [-14], [-30], [-8], [66], [-72], [32], [26], [-30], [-56], [54], [-28], [14], [54], [-14], [30], [-38], [30], [42], [-18], [62], [36], [62], [42], [-66], [-34], [-30], [56], [-58], [-18], [-8], [6], [-82], [14], [8], [6], [-78], [82], [-30], [28], [60], [86], [-84], [58], [-30], [50], [-2], [10], [-78], [90], [52], [66], [-36], [58], [80], [-54], [-30], [62], [-70], [-78], [-14], [-86], [-6], [30], [-42], [-30], [-12], [-26], [78], [-78], [-54], [-10], [-94], [18], [64], [74], [-60], [68], [-72], [-66], [40], [-38], [18], [12], [2], [6], [50], [-6], [54], [-20], [78], [-10], [96], [46], [-38], [32], [42], [26], [78], [-90], [-46], [-30], [-46], [34], [54], [68], [90], [-14], [-54], [28], [-18], [42], [-56], [-50], [18], [-76], [54], [2], [18], [54], [8], [26], [-78], [18], [58], [54], [42], [30], [58], [48], [96], [74], [-18], [18], [18], [26], [36]]
